/*
Copyright (c) 2007, Dennis M. Sosnoski
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of JiBX nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.jibx.binding.generator;

import java.util.Iterator;

import org.jibx.binding.model.BindingElement;
import org.jibx.binding.model.NamespaceElement;

/**
 * External data for a binding definition. This tracks references to other
 * bindings, along with the associated namespace information.
 */
public class BindingHolder extends HolderBase
{
    private final BindingElement m_binding;
    
    /**
     * Constructor.
     * 
     * @param uri (<code>null</code> if no-namespace binding)
     * @param index binding index number
     */
    public BindingHolder(String uri, int index) {
        super(uri);
        m_binding = new BindingElement();
        if (uri != null) {
            NamespaceElement ns = new NamespaceElement();
            ns.setDefaultName("elements");
            ns.setUri(uri);
            ns.setPrefix("ns" + index);
            m_binding.addTopChild(ns);
            m_binding.addNamespaceDecl("tns", uri);
        }
    }
    
    public BindingElement getBinding() {
        return m_binding;
    }

    /* (non-Javadoc)
     * @see org.jibx.binding.generator.HolderBase#addNamespaceDecl(java.lang.String, java.lang.String)
     */
    protected void addNamespaceDecl(String prefix, String uri) {
        m_binding.addNamespaceDecl(prefix, uri);
    }

    /**
     * Handle references from this binding to other bindings. This adds
     * namespace declarations to the root element of the binding for the
     * namespaces of the referenced bindings.
     */
    public void fixReferences() {
        for (Iterator iter = getReferences().iterator(); iter.hasNext();) {
            getPrefix(((BindingHolder)iter.next()).getNamespace());
        }
    }
}