/*
Copyright (c) 2007, Dennis M. Sosnoski
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of JiBX nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.jibx.binding.generator;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Set of unique names for a context. This assures uniqueness as names are added
 * to the set.
 */
public class UniqueNameSet
{
    /** Set of names used. */
    private final Set m_nameSet;
    
    /**
     * Constructor.
     */
    public UniqueNameSet() {
        m_nameSet = new HashSet();
    }
    
    /**
     * Add name to set. If the supplied name is already present, it is modified
     * by appending a numeric suffix starting at '1' and going as high as
     * necessary to obtain a unique name.
     *
     * @param base name to try adding
     * @return assigned name
     */
    public String add(String base) {
        String name = base;
        int index = 1;
        while (m_nameSet.contains(name)) {
            name = base + index++;
        }
        m_nameSet.add(name);
        return name;
    }
    
    /**
     * Get iterator for names in set.
     *
     * @return iterator
     */
    public Iterator iterator() {
        return m_nameSet.iterator();
    }
}