/*
Copyright (c) 2005, Dennis M. Sosnoski
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.
 * Neither the name of JiBX nor the names of its contributors may be used
   to endorse or promote products derived from this software without specific
   prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package org.jibx.binding.schema.types;

/**
 * Character sequence based on a character array. Instances support direct
 * access to the underlying array for more efficient operation.
 * 
 * Created Jul 24, 2005
 * @author Dennis M. Sosnoski
 */
public class CharArraySequence implements ICharSequence
{
    /** Starting offset of sequence. */
    private int m_start;
    
    /** Length of sequence. */
    private int m_length;
    
    /** Backing array. */
    private char[] m_array;
    
    /**
     * Constructor.
     * 
     * @param array backing array
     * @param start starting offset in array
     * @param length number of characters in sequence
     */
    public CharArraySequence(char[] array, int start, int length) {
        m_array = array;
        m_start = start;
        m_length = length;
    }

    /**
     * Get backing array.
     * 
     * @return array
     */
    public char[] getArray() {
        return m_array;
    }

    /**
     * Set backing array.
     * 
     * @param array array
     */
    public void setArray(char[] array) {
        m_array = array;
    }

    /**
     * Get start offset in array.
     * 
     * @return start offset
     */
    public int getStart() {
        return m_start;
    }

    /**
     * Set number of characters in sequence.
     * 
     * @param length number of characters
     */
    protected void setLength(int length) {
        m_length = length;
    }

    /**
     * Set range in array.
     * 
     * @param start start offset
     * @param length number of characters
     */
    protected void setRange(int start, int length) {
        m_start = start;
        m_length = length;
    }

    /**
     * Set backing array and range.
     * 
     * @param array backing array
     * @param start start offset
     * @param length number of characters
     */
    protected void setStart(char[] array, int start, int length) {
        m_array = array;
        m_start = start;
        m_length = length;
    }

    /* (non-Javadoc)
     * @see org.jibx.binding.schema.types.ICharSequence#charAt(int)
     */
    public char charAt(int index) {
        return m_array[m_start+index];
    }

    /* (non-Javadoc)
     * @see org.jibx.binding.schema.types.ICharSequence#length()
     */
    public int length() {
        return m_length;
    }

    /* (non-Javadoc)
     * @see org.jibx.binding.schema.types.ICharSequence#subSequence(int, int)
     */
    public ICharSequence subSequence(int start, int end) {
        return new CharArraySequence(m_array, start, end);
    }
}